--[[
Interface: 1.3.0.1 b4007

Copyright (C) Tommy , 2021

Author: Tommy
Date: 02.08.2021
Version: 1.0.0.0

History:
V 1.0.0.0 @ 02.08.2021 - Release Version for FS19.

Important:
Free for use in other mods - no permission needed.

frei verwendbar - keine erlaubnis nötig
]]


TurnLightExtension = {}


function TurnLightExtension.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Lights, specializations)
end

function TurnLightExtension.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", TurnLightExtension)
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", TurnLightExtension)
end

function TurnLightExtension.registerFunctions(vehicleType)
	
end

function TurnLightExtension:onLoad(savegame)
	self.spec_TurnLightExtension = {}
	local spec = self.spec_TurnLightExtension
	spec.last_onoff_state = 0
	spec.lastTurnStatus = -1
	
	local actionsKey = "vehicle.TurnLightExtension.insideActions"
	
	-- Leggi tutti i parametri insideActions dall'xml	
	if hasXMLProperty(self.xmlFile, actionsKey) then
		local key = ""
		spec.statusTurnLights = 0
		spec.statusInsideVehicle = 0
		spec.insideActions = {}
		spec.insideActions.LeftTurn = {}
		spec.insideActions.RightTurn = {}
		
		spec.insideActions.dynaRightTurn = {}
		spec.insideActions.dynaLeftTurn = {}
		
		key = string.format("%s.actionLeftTurn", actionsKey)
		if hasXMLProperty(self.xmlFile, key) then
			spec.insideActions.LeftTurn.animationName = getXMLString(self.xmlFile, key .. "#animationName")
			spec.insideActions.LeftTurn.animationSpeed = getXMLFloat(self.xmlFile, key .. "#animationSpeed")
		end
		
		key = string.format("%s.actionRightTurn", actionsKey)
		if hasXMLProperty(self.xmlFile, key) then
			spec.insideActions.RightTurn.animationName = getXMLString(self.xmlFile, key .. "#animationName")
			spec.insideActions.RightTurn.animationSpeed = getXMLFloat(self.xmlFile, key .. "#animationSpeed")
		end
		
		key = string.format("%s.dynaRightTurn", actionsKey)
		if hasXMLProperty(self.xmlFile, key) then
			spec.insideActions.dynaRightTurn.animationName = getXMLString(self.xmlFile, key .. "#animationName")
			spec.insideActions.dynaRightTurn.syncOffset = getXMLFloat(self.xmlFile, key .. "#syncOffset")
			spec.insideActions.dynaRightTurn.cosOffset = getXMLFloat(self.xmlFile, key .. "#cosOffset")
			spec.insideActions.dynaRightTurn.lastToggleOnOff = 0
		end
		
		key = string.format("%s.dynaLeftTurn", actionsKey)
		if hasXMLProperty(self.xmlFile, key) then
			spec.insideActions.dynaLeftTurn.animationName = getXMLString(self.xmlFile, key .. "#animationName")
			spec.insideActions.dynaLeftTurn.syncOffset = getXMLFloat(self.xmlFile, key .. "#syncOffset")
			spec.insideActions.dynaLeftTurn.cosOffset = getXMLFloat(self.xmlFile, key .. "#cosOffset")
			spec.insideActions.dynaLeftTurn.lastToggleOnOff = 0
		end
	end
end

function TurnLightExtension:onUpdate(savegame)
	
	local spec = self.spec_TurnLightExtension	
	
	-- se lo stato delle frecce è cambiato
	if spec.lastTurnStatus ~= self.spec_lights.turnLightState then
		local leftTurnMovement = 0
		local rightTurnMovement = 0
		
		-- in caso di cambio stato, registra l'evento avvenuto e usalo nella decisione dell'azione
		event = self.spec_lights.turnLightState
		
		-- decide l'azione da eseguire in base allo stato attuale e all'evento avvenuto
		if spec.statusTurnLights == 0 then -- turn lights are off
			if event == 1 then -- se arriva evento freccia sx
				-- cerco il Left e accendo
				leftTurnMovement = 1
				spec.statusTurnLights = 1
			elseif event == 2 then -- se arriva evento freccia dx
				-- cerco il Right e accendo
				rightTurnMovement = 1
				spec.statusTurnLights = 2
			elseif event == 3 then -- se arriva evento freccia hazard
				-- cerco il Left e il Right e accendo
				leftTurnMovement = 1
				rightTurnMovement = 1
				spec.statusTurnLights = 3
			end
		elseif spec.statusTurnLights == 1 then -- stato freccia sx accesa
			if event == 1 then -- se arriva evento freccia sx
				-- cerco il Left e spengo
				leftTurnMovement = -1
				spec.statusTurnLights = 0
			elseif event == 2 then -- se arriva evento freccia dx
				-- cerco il Right e accendo
				leftTurnMovement = -1
				rightTurnMovement = 1
				spec.statusTurnLights = 2
			elseif event == 3 then -- se arriva evento freccia hazard
				-- cerco il Left e il Right e accendo
				rightTurnMovement = 1
				spec.statusTurnLights = 3
			elseif event == 0 then -- se arriva evento 0	
				-- cerco il Left e spengo
				leftTurnMovement = -1
				spec.statusTurnLights = 0
			end
		elseif spec.statusTurnLights == 2 then -- stato freccia dx accesa
			if event == 1 then -- se arriva evento freccia sx
				-- cerco il Left e spengo
				leftTurnMovement = 1
				rightTurnMovement = -1			
				spec.statusTurnLights = 1
			elseif event == 2 then -- se arriva evento freccia dx
				-- cerco il Right e accendo
				rightTurnMovement = -1			
				spec.statusTurnLights = 0
			elseif event == 3 then -- se arriva evento freccia hazard
				-- cerco il Left e accendo
				leftTurnMovement = 1
				spec.statusTurnLights = 3
			elseif event == 0 then -- se arriva evento 0	
				-- cerco il Left e spengo
				rightTurnMovement = -1			
				spec.statusTurnLights = 0
			end
		elseif spec.statusTurnLights == 3 then -- stato freccia hazard accese
			if event == 1 then -- se arriva evento freccia sx
				-- cerco il Right e spengo
				rightTurnMovement = -1			
				spec.statusTurnLights = 1
			elseif event == 2 then -- se arriva evento freccia dx
				-- cerco il Right e accendo
				leftTurnMovement = -1
				spec.statusTurnLights = 2
			elseif event == 3 then -- se arriva evento freccia hazard
				-- cerco il Left eil Right spengo
				leftTurnMovement = -1
				rightTurnMovement = -1
				spec.statusTurnLights = 0
			elseif event == 0 then -- se arriva evento 0	
				-- cerco il Left eil Right spengo
				leftTurnMovement = -1
				rightTurnMovement = -1
				spec.statusTurnLights = 0
			end
		end
		-- esegue l'animazione in base all'azione definita sopra
		if leftTurnMovement == 1 then
			self:playAnimation(spec.insideActions.LeftTurn.animationName, spec.insideActions.LeftTurn.animationSpeed, self:getAnimationTime(spec.insideActions.LeftTurn.animationName), true);
		elseif leftTurnMovement == -1 then
			self:playAnimation(spec.insideActions.LeftTurn.animationName, -spec.insideActions.LeftTurn.animationSpeed, self:getAnimationTime(spec.insideActions.LeftTurn.animationName), true);
			self:playAnimation(spec.insideActions.dynaLeftTurn.animationName, -9999, self:getAnimationTime(spec.insideActions.dynaLeftTurn.animationName), true);
		end
		
		if rightTurnMovement == 1 then
			self:playAnimation(spec.insideActions.RightTurn.animationName, spec.insideActions.RightTurn.animationSpeed, self:getAnimationTime(spec.insideActions.RightTurn.animationName), true);
		elseif rightTurnMovement == -1 then
			self:playAnimation(spec.insideActions.RightTurn.animationName, -spec.insideActions.RightTurn.animationSpeed, self:getAnimationTime(spec.insideActions.RightTurn.animationName), true);
			self:playAnimation(spec.insideActions.dynaRightTurn.animationName, -9999, self:getAnimationTime(spec.insideActions.dynaRightTurn.animationName), true);
		end
	end
	
	-- salva l'ultimo stato delle frecce 
	spec.lastTurnStatus = self.spec_lights.turnLightState
	
	-- esegue l'animazione per il lampeggio delle frecce in base allo scorrere del tempo con un certo ritardo (syncOffset) e duty cycle (cosOffset)
	if spec.statusTurnLights == 1 then -- stato freccia sx accesa
		local onoff = math.cos(7*(getShaderTimeSec()-spec.insideActions.dynaLeftTurn.syncOffset)) + spec.insideActions.dynaLeftTurn.cosOffset -- 0.2
		if onoff >= 0 then
			if spec.insideActions.dynaLeftTurn.lastToggleOnOff < 0 then	
				self:playAnimation(spec.insideActions.dynaLeftTurn.animationName, 1, self:getAnimationTime(spec.insideActions.dynaLeftTurn.animationName), true);
			end
		elseif onoff <= 0 then
			if spec.insideActions.dynaLeftTurn.lastToggleOnOff > 0 then
				self:playAnimation(spec.insideActions.dynaLeftTurn.animationName, -1, self:getAnimationTime(spec.insideActions.dynaLeftTurn.animationName), true);
			end
		end
		spec.insideActions.dynaLeftTurn.lastToggleOnOff = onoff
	elseif spec.statusTurnLights == 2 then -- stato freccia dx accesa
		local onoff = math.cos(7*(getShaderTimeSec()-spec.insideActions.dynaRightTurn.syncOffset)) + spec.insideActions.dynaRightTurn.cosOffset -- 0.2
		if onoff >= 0 then
			if spec.insideActions.dynaRightTurn.lastToggleOnOff < 0 then
				self:playAnimation(spec.insideActions.dynaRightTurn.animationName, 1, self:getAnimationTime(spec.insideActions.dynaRightTurn.animationName), true);
			end
		elseif onoff <= 0 then
			if spec.insideActions.dynaRightTurn.lastToggleOnOff > 0 then
				self:playAnimation(spec.insideActions.dynaRightTurn.animationName, -1, self:getAnimationTime(spec.insideActions.dynaRightTurn.animationName), true);
			end
		end
		spec.insideActions.dynaRightTurn.lastToggleOnOff = onoff	
	elseif spec.statusTurnLights == 3 then -- stato freccia hazard accese
		local onoff = math.cos(7*(getShaderTimeSec()-spec.insideActions.dynaLeftTurn.syncOffset)) + spec.insideActions.dynaLeftTurn.cosOffset -- 0.2
		if onoff >= 0 then
			if spec.insideActions.dynaLeftTurn.lastToggleOnOff < 0 then	
				self:playAnimation(spec.insideActions.dynaLeftTurn.animationName, 1, self:getAnimationTime(spec.insideActions.dynaLeftTurn.animationName), true);
			end
		elseif onoff <= 0 then
			if spec.insideActions.dynaLeftTurn.lastToggleOnOff > 0 then
				self:playAnimation(spec.insideActions.dynaLeftTurn.animationName, -1, self:getAnimationTime(spec.insideActions.dynaLeftTurn.animationName), true);
			end
		end
		spec.insideActions.dynaLeftTurn.lastToggleOnOff = onoff
		
		onoff = math.cos(7*(getShaderTimeSec()-spec.insideActions.dynaRightTurn.syncOffset)) + spec.insideActions.dynaRightTurn.cosOffset -- 0.2
		if onoff >= 0 then
			if spec.insideActions.dynaRightTurn.lastToggleOnOff < 0 then	
				self:playAnimation(spec.insideActions.dynaRightTurn.animationName, 1, self:getAnimationTime(spec.insideActions.dynaRightTurn.animationName), true);
			end
		elseif onoff <= 0 then
			if spec.insideActions.dynaRightTurn.lastToggleOnOff > 0 then
				self:playAnimation(spec.insideActions.dynaRightTurn.animationName, -1, self:getAnimationTime(spec.insideActions.dynaRightTurn.animationName), true);
			end
		end
		spec.insideActions.dynaRightTurn.lastToggleOnOff = onoff	
	end
end




